from datetime import datetime

def current_time():
    return datetime.now()  # current date and time

def get_time():
    return current_time().strftime("%H:%M:%S")

def get_full_date():
    return current_time().strftime("%Y%m%d")

def get_full_data_time():
    return current_time().strftime("%Y%m%d_%H%M%S")




# https://medium.com/@joel.barmettler/how-to-upload-your-python-package-to-pypi-65edc5fe9c56
import pandas as pd
from scipy import stats
import os, sys

def choose_multiple_values_in_column(df, colName, values):
    '''
    Return the dataframe (df) with selected string (values) in colName
    @param df: pandas dataframe
    @param colName: which column
    @param values: what are the values of intrest
    @return: dataframe (df) with selected string (values) in colName
    '''
    return df[df[colName].str.contains('|'.join(values))]

def sclmns(df, s, not_in=False):
    '''
    Return all the columns that (not) contain the names in s.
    :param df: pandas dataframe.
    :param s: string(s) the column(s) contains.
    :param not_in: all columns except the ones selected.
    :return: list of columns.
    '''
    clmns = []
    if not_in:
        # all_clmns = list(df.columns)
        all_clmns = df.columns
        for i in s:
            # all_clmns.remove(i)
            # all_clmns = [x for x in all_clmns if not x.__contains__(i)]
            all_clmns = all_clmns[~all_clmns.str.contains(i)]
        clmns = list(all_clmns)
        # clmns = all_clmns.copy()
    else:
        for i in s:
            clmns = clmns + list(df.columns[df.columns.str.contains(i)])
    return clmns

def calculate_corr_with_pvalues(df, method = 'pearsonr'):
    '''
    Calculate the correlation between the columns of dataframe and return values with stars.
    :param df: dataframe
    :param method: pearsonr/ spearman
    :return: pvalues, corr_with_stars
    '''
    df = df.dropna()._get_numeric_data()
    dfcols = pd.DataFrame(columns=df.columns)
    pvalues = dfcols.transpose().join(dfcols, how='outer')

    rho = df.corr()

    for r in df.columns:
        for c in df.columns:
            if method == 'pearsonr':
                pvalues[r][c] = round(stats.pearsonr(df[r], df[c])[1], 4)
            elif method == 'spearman':
                pvalues[r][c] = round(stats.spearmanr(df[r], df[c])[1], 4)
            elif method == 'reg':
                slope, intercept, rho[r][c], pvalues[r][c], std_err = stats.linregress(x=df[r],y= df[c])

    rho = rho.round(2)
    pval = pvalues
    # create three masks
    r1 = rho.applymap(lambda x: '{}*'.format(x))
    r2 = rho.applymap(lambda x: '{}**'.format(x))
    r3 = rho.applymap(lambda x: '{}***'.format(x))
    # apply them where appropriate
    rho = rho.mask(pval <= 0.05, r1)
    rho = rho.mask(pval <= 0.01, r2)
    rho = rho.mask(pval <= 0.001, r3)

    return pvalues, rho


def save_latex(data_path, df, fn, ix = True):
    '''
    Save dataframe with latex file
    :param data_path: were to open the .tex file
    :param df: pandas dataframe
    :param fn: filename
    :param ix: save index
    :return:
    '''
    with open(data_path + '%s.tex' %fn, 'w') as tf:
        if type(df) != pd.core.frame.DataFrame:
            df = pd.DataFrame(df)
        tf.write(df.to_latex(float_format=lambda x: '%.3f' % x, index=ix))
        df.to_csv(data_path + '%s.csv' % fn)


def ttest_or_mannwhitney(y1,y2):
    '''
    Check if y1 and y2 stand the assumptions for ttest and if not preform mannwhitney
    :param y1: 1st sample
    :param y2: 2nd sample
    :return: s, pvalue, ttest - True/False
    '''
    ttest = False

    # assumptions for t-test
    # https://pythonfordatascience.org/independent-t-test-python/#t_test-assumptions
    ns1, np1 = stats.shapiro(y1)  # test normality of the data
    ns2, np2 = stats.shapiro(y2)  # test noramlity of the data
    ls, lp = stats.levene(y1, y2)  # test that the variance behave the same
    if (lp > .05) & (np1 > .05) & (np2 > .05):
        ttest = True
        s, p = stats.ttest_ind(y1, y2)
    else:
        s, p = stats.mannwhitneyu(y1, y2)

    return s, p, ttest

def new_dir(dirPath):
    if not os.path.isdir(dirPath):
        os.mkdir(dirPath)

def progress(i, N, text = None):
    '''
    Update progress in the same line
    :param i: current progress
    :param N: total count
    :param text: description text
    :return:
    '''
    sys.stdout.write('/r')
    if text is None:
        sys.stdout.write('Frame {0}/{1} added to video'.format(i, N))
    else:
        sys.stdout.write(text + '{0}/{1}'.format(i, N))
    sys.stdout.flush()


import PIL.Image

def main():
    path = input("Enter the path to the image fiel : \n")
    try:
        image = PIL.Image.open(path)
    except:
        print(path, "Unable to find image ");

# ASCII_CHARS = ["@", "#", "$", "%", "?", "*", "+", ";", ":", ",", "."]
# ASCII_CHARS = ["@", "#", "$", "%", "?", "*", " ", " ", " ", " ", " "]
ASCII_CHARS = ["*", "*", "*", "*", "*", "*", " ", " ", " ", " ", " "]

def resize(image, new_width = 20):
    width, height = image.size
    new_height = int(new_width * height / width)
    return image.resize((new_width, new_height))

### for batman logo
# def resize(image, new_width = 70):
#     width, height = image.size
#     # new_height = int(new_width * height / width)
#     new_height = 25
#     return image.resize((new_width, new_height))

def to_greyscale(image):
    return image.convert("L")

def pixel_to_ascii(image):
    pixels = image.getdata()
    ascii_str = "";
    for pixel in pixels:
        ascii_str += ASCII_CHARS[pixel//25];
    return ascii_str

def main():
    # path = input("Enter the path to the image fiel : \n")
    path = 'thammer.jpg'
    # path = 'car.jpg'
    # path = 'batman.jpg'
    try:
        image = PIL.Image.open(path)
    except:
        print(path, "Unable to find image ")
    #resize image
    image = resize(image);
    #convert image to greyscale image
    greyscale_image = to_greyscale(image)
    # convert greyscale image to ascii characters
    ascii_str = pixel_to_ascii(greyscale_image)
    img_width = greyscale_image.width
    ascii_str_len = len(ascii_str)
    ascii_img=""
    #Split the string based on width  of the image
    for i in range(0, ascii_str_len, img_width):
        ascii_img += ascii_str[i:i+img_width] + "\n"
    print(ascii_img)
    #save the string to a file
    with open("ascii_image.txt", "w") as f:
        f.write(ascii_img);

def signature():
    print('''
       ** 
        ** 
         **    
          **     *     
           **   ***    
            ** ******   
             ********* 
             *      *** 
            ****  **** 
           *****  ***  
            ****  **   
             ******     
              ****     
               **
    ''')

if __name__ == '__main__':
    # main()
    signature()


from scipy import stats
import plotly.graph_objs as go
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import RectangleSelector
import matplotlib.mlab as mlab
import plotly
import plotly.tools as tls
from tkinter import *
import os


def plotly_reg(x, y, fig, rc, clr, description_txt='', x_var_name ='', y_var_name ='',
               xaxis = None, yaxis = None, xlbl ='', ylbl ='',
               scatter_plot = 0, range=None, ax_mpl=None, mpl_clr=None):
    '''Doing regression plot using plotly
    rc = regression comtour'''
    # Calculating the regression line
    slope, intercept, r_value, p_value, std_err = stats.linregress(x[~np.isnan(y)], y[~np.isnan(y)])
    y_reg = slope * x + intercept
    reg_analysis = 'r = %.2f p = %.2f' % (r_value, p_value)
    reg_equation = y_var_name + ' = %.2f*' % (slope)
    reg_equation += x_var_name + '%+.2f' % (intercept)
    temp_text = description_txt + ', ' + reg_analysis + ', ' + reg_equation

    # Plotting the regression line
    line = go.Scatter(x=x, y=y_reg, mode='lines+text', line=dict(width=2, color=clr),
                      name=temp_text)
    fig.append_trace(line, rc[0], rc[1])

    # Plotting scatter of the data
    if scatter_plot == 1:
        trace = go.Scatter(x=x, y=y, mode='markers', marker=dict(color='rgb'))
        fig.append_trace(trace, rc[0], rc[1])

    if yaxis != None and xaxis != None:
        fig['layout'][yaxis].update(title=ylbl)
        fig['layout'][xaxis].update(title=xlbl)

    if ax_mpl != None:
        ax_mpl.plot(x, y_reg, label=description_txt, color=mpl_clr)

    return slope, intercept, r_value, p_value, std_err


def format_axis(ax, xlbl, ylbl, ind = None):
    '''
    format axis of matplotlib
    example: format_axis(ax[0], ylbl='Entanglement', xlbl='Questions order')
    :param ind:
    :return:
    '''
    ax.set_ylabel(ylbl)
    ax.set_xlabel(xlbl)
    ax.legend()

    if ind != None:
        ax.set_xticks(ind)
        ax.tick_params(axis='both', which='major', labelsize=20)
        ax.set_xticklabels(ind)
        for label in ax.get_xmajorticklabels():
            label.set_rotation(30)
            label.set_horizontalalignment("right")


def show_message(file_name):
    '''Prints text message from txt file'''
    fl1 = open(file_name, 'r')
    f_str = fl1.read()
    fl1.close()
    # Output a message
    root = Tk()
    w = Label(root, text=f_str)
    w.pack()
    root.mainloop()


def save_maxfig(fig, fig_name, save_plotly=False, transperent = False, frmt='png', resize=None):
    '''
    Save figure in high resolution
    example:
    save_maxfig(fig,fig_name, save_plotly=True, transperent=True, frmt='eps', resize=[18,6])
    :param fig: figure
    :param fig_name: file name
    :param save_plotly: save to plotly also
    :param transperent: no background
    :param frmt: file format
    :param resize: size
    :return:
    '''
    # matplotlib.rcParams.update({'font.size': 40})
    fig.set_size_inches(12, 12)
    if resize != None:
        fig.set_size_inches(resize[0], resize[1])
    fig_name+='.'+frmt
    plt.savefig(fig_name, dpi=300, format=frmt)

    if save_plotly :
        # offline plotting
        plotly_fig = tls.mpl_to_plotly(fig)
        plotly_fig_name = + fig_name + '.html'
        plotly_fig['layout'].update(height=1500, width=800, showlegend=False)
        plotly.offline.plot(plotly_fig, filename=plotly_fig_name, auto_open=False)  # offline ploty


def manual_matplotlib_crop(fullImage):
    '''
    Manually crop part of the image.
    @param fullImage: Image to crop
    @return: the cropped image.
    '''

    def line_select_callback(eclick, erelease):
        '''
        Get x1, y1 when cliked and x2,y2 when released.
        '''
        x1, y1 = eclick.xdata, eclick.ydata
        x2, y2 = erelease.xdata, erelease.ydata

    def toggle_selector(event):
        '''
        (de) Activate the selector.
        '''
        print(' Key pressed.')
        if event.key in ['Q', 'q'] and toggle_selector.RS.active:
            print(' RectangleSelector deactivated.')
            toggle_selector.RS.set_active(False)
        if event.key in ['A', 'a'] and not toggle_selector.RS.active:
            print(' RectangleSelector activated.')
            toggle_selector.RS.set_active(True)



    fig, ax = plt.subplots(1, 1, figsize=(15, 10))
    ax.imshow(fullImage)

    toggle_selector.RS = RectangleSelector(ax, line_select_callback,
                                           drawtype='box', useblit=True,
                                           button=[1,3],
                                           minspanx=5, minspany=5,
                                           spancoords='pixels',
                                           interactive=True)

    plt.connect('key_pressed_event', toggle_selector)
    plt.show()
    cropCoords = [int(x) for x in toggle_selector.RS.extents]

    croopedIm = fullImage[cropCoords[2]:cropCoords[3], cropCoords[0]:cropCoords[1]]

    return croopedIm


# if __name__ == '__main__':
#
#     im = plt.imread('car.png')
#     cIm = manual_matplotlib_crop(im)
#     plt.imshow(cIm)
#     plt.show()



import smtplib
import os
from sys import platform


def sendemail(from_addr, to_addr_list, cc_addr_list,
              subject, message,
              login, password,
              smtpserver='smtp.gmail.com:587'):
    '''
    example:
    sendemail(from_addr    = 'user1@gmail.com',
          to_addr_list = ['user2@gmail.com'],
          cc_addr_list = ['user2@gmail.com'],
          subject      = 'python sending email trail',
          message      = 'email from a python function',
          login        = 'user1@gmail.com',
          password     = 'userq's password')
    '''
    header = 'From: %s \n' % from_addr
    header += 'To: %s\n' % ', '.join(to_addr_list)
    header += 'Cc: %s\n' % ', '.join(cc_addr_list)
    header += 'Subject: %s\n' % subject
    message = header + message

    server = smtplib.SMTP(smtpserver)
    server.starttls()
    server.login(login, password)
    problems = server.sendmail(from_addr, to_addr_list, message)
    server.quit()

def sound_notification():
    '''
    Sound notification.
    need to install: sudo apt install speech-dispatcher
    need to install: sudo apt install sox
    :return:
    '''
    if platform == "linux" or platform == "linux2":
        duration = .5  # second
        freq = 440  # Hz
        os.system('play --no-show-progress --null --channels 1 synth %s sine %f' % (duration, freq))
        # os.system('spd-say "your program has finished"')
    elif platform == "win32" or 'darwin':
        import winsound
        duration = 500  # millisecond
        freq = 440  # Hz
        winsound.Beep(freq, duration)

# sound_notification()


import dash
from dash import html, dcc, dash_table
import pandas as pd
import plotly_express as px
import matplotlib.pyplot as plt
import numpy as np

# dfPath = '.csv'
# imPath = 'car.png'
# df = pd.read_csv(dfPath, index_col=0)


df = pd.DataFrame(columns=['A', 'B'], data = [[1,2],[3,4]])

images = ['../car.png', '../car.png']
for i, im in enumerate(images):
    im_ = plt.imread(im)
    im_ = im_.reshape(*im_.shape,1)
    if i ==0:
        array_of_images = im_.copy()
    else:
        array_of_images = np.concatenate((array_of_images, im_), axis=3)


### plot multiple images (can also plot only one)

if array_of_images.shape[-1] < 2:
    array_of_images.reshape(*array_of_images.shape,1)

figx  = px.imshow(array_of_images, facet_col=3, facet_col_wrap=4, binary_format='png')#, binary_string=True)

### Change the titles of the images
subTitles = list('ab')
for i, annot in enumerate(subTitles):
    figx.layout.annotations[i]['text'] = annot

figx.update_layout(
    width=2000,
    height=1000,
    margin = dict(
        l=1,
        r=1,
        b=1,
        t=1,
        pad=1
    ))

app = dash.Dash(__name__)

app.layout = html.Div([
    html.H4(children='Title'),
    dash_table.DataTable(
        id='table',
        columns = [{"name":i, "id":i} for i in df.columns],
        data=df.to_dict('records'),
        style_cell=dict(textAlign='left'),
        style_header=dict(backgroundColor='paleturquoise'),
        style_data=dict(backgroundColor='lavender'),
    ),
    dcc.Graph(
        id='example-graph',
        figure=figx
    )
])

if __name__ == '__main__':
    app.run_server(debug=True, use_reloader=False)


from setuptools import setup

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(name='torrs_hammers',
      version='0.2.2',
      description='useful tools',
      url='https://github.com/PyTorr/torrs_hammers/archive/refs/tags/v02.tar.gz',
      author='PyTorr',
      author_email='',
      license='MIT',
      packages=['torrs_hammers'],
      zip_safe=False,
      long_description=long_description,
      long_description_content_type='text/markdown'
      )
